/* cwd.c -- cwd builtin */

/* Copyright (C) 2003 Mike Thielman 
 *
 * This file is part of pr0nsh, the Porn Shell.
 *
 * Pr0nsh is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2, or (at your option) any later
 * version.
 *
 * Pr0nsh is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Pr0nsh; see the file COPYING.  If not, write to the Free Software
 * Foundation, 59 Temple Place, Suite 330, Boston, MA 02111 USA. */

#include <stdio.h>
#include <pwd.h>
#include "shell/dir.h"
#include "shell/redir.h"
#include "util/debug.h"
#include "util/llist.h"
#include "util/symbol.h"
#include "shell/shell.h"

int _builtin_cwd(LinkedList *list)
{
  int n;
  LinkedList *newdir;
  char *_newdir;
  char *p,*arg;

  n = numEntriesll(list);
  if(n == 0) {
    if(VarExists("HOME")) { 
      mychdir(VarValue("HOME"));
      freell(getPWD());
      setPWD(_getNewDir(VarValue("HOME"))); 
    }
    else {
      mychdir("/");
      free(getPWD());
      setPWD(NULL);
    }
  } else if(n == 1) {
    newdir = _getNewDir(list->s);
    _newdir = dir_ll_to_str(newdir);
    if(dirExists(_newdir)) {
      mychdir(_newdir);
      freell(getPWD());
      setPWD(newdir);
    } else {
      free(newdir);
      outputerr("Error: directory \"%s\" doesn't exist\n", _newdir);
      return RET_ERROR;
    }
    free(_newdir);
  } else {
    outputerr("Invalid number of arguments to: cd\n");
    return RET_ERROR;
  }
  return RET_SUCCESS;
}
