#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include "parse/ast.h"
#include "shell/alias.h"
#include "pr0nsh.h"

static int _interactive = 1;    // If shell is interactive
static int _pid = 0;            // PID of currently running process
static char *_curcmd = NULL;    // Command-line of currently running process
static char *_exec_next = NULL; // If not NULL, this should be executed next
                                // rather than what is on STDIN
static ASTCommand *_ast = NULL; // AST of currently running process
static Alias *_aliases = NULL;  // List of currently defined aliases
static FPList *_fplist = NULL;  // List of fp's to execute commands from

void execFromFP(FILE *fp)
{
  FPList *tmp;

  tmp = (FPList *) emalloc(sizeof(FPList));
  tmp->next = _fplist;
  tmp->fp = fp;
  _fplist = tmp;
}

FILE *getNextFP(void)
{
  if(_fplist) return _fplist->fp;
  return NULL;
}

void closeCurFP(void)
{
  FPList *tmp;
  
  if(_fplist) {
    fclose(_fplist->fp);
    tmp = _fplist->next;
    free(_fplist);
    _fplist = tmp;
  }
}

void setCurrentAliases(Alias *alias)
{
  Alias *tmp;

  tmp = _aliases;
  _aliases = copyAlias(alias);
//  freeAlias(tmp);
}

Alias *getCurrentAliases(void)
{
  return _aliases;
}

void setCurrentAST(ASTCommand *ast)
{
  freeAST(_ast);
  _ast = copyAST(ast);
}

ASTCommand *getCurrentAST(void)
{
  return _ast;
}

void setExecNext(char *cmd)
{
  free(_exec_next);
  if(cmd) _exec_next = strdup(cmd);
  else _exec_next = NULL;
}

char *getExecNext(void)
{
  return _exec_next;
}

void setCurCMD(char *cmd)
{
  free(_curcmd);
  if(cmd) _curcmd = strdup(cmd);
  else _curcmd = NULL;
}

char *getCurCMD(void)
{
  return _curcmd;
}

void setInteractive(int n)
{
  _interactive = n;
}

int isInteractive(void)
{
  return _interactive;
}

void setCurrentPID(pid_t pid)
{
  _pid = pid;
}

pid_t getCurrentPID(void)
{
  return _pid;
}
