/* init.c -- Initialize various shell variables and functions. */

/* Copyright (C) 2003 Mike Thielman 
 *
 * This file is part of pr0nsh, the Porn Shell.
 *
 * Pr0nsh is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2, or (at your option) any later
 * version.
 *
 * Pr0nsh is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Pr0nsh; see the file COPYING.  If not, write to the Free Software
 * Foundation, 59 Temple Place, Suite 330, Boston, MA 02111 USA. */

#include <stdio.h>
#include "builtins/builtins.h"
#include "util/symbol.h"
#include "shell/shell.h"
#include "shell/redir.h"

static struct {
  char *name;
  int (*func)();
} builtins[] = {
  "builtins",   _builtin_builtins,
  "exit",       _builtin_exit,
  "pwd",        _builtin_pwd,
  "cd",         _builtin_cwd,
  "vars",       _builtin_vars,
  "echo",       _builtin_echo,
  "history",    _builtin_history,
  "alias",      _builtin_alias,
  "jobs",       _builtin_jobs,
  "bg",         _builtin_bg,
  "fg",         _builtin_fg,
  "export",     _builtin_export,
  "which",      _builtin_which,
  "source",     _builtin_source,
  0, 0
};

init()
{
  int i;
  Symbol *s;

  setSTDINfd(STREAM_STDIN);
  setSTDOUTfd(STREAM_STDOUT);
  setSTDERRfd(STREAM_STDERR);
  
  for (i = 0; builtins[i].name; i++) {
    s = install(builtins[i].name, _BLTIN, NULL);
    s->ptr = builtins[i].func;
  }
  initShell();
}
