/* shell.c -- Shell-specific commands that have no other place yet */

/* Copyright (C) 2003 Mike Thielman 
 *
 * This file is part of pr0nsh, the Porn Shell.
 *
 * Pr0nsh is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2, or (at your option) any later
 * version.
 *
 * Pr0nsh is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Pr0nsh; see the file COPYING.  If not, write to the Free Software
 * Foundation, 59 Temple Place, Suite 330, Boston, MA 02111 USA. */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include "util/llist.h"
#include "util/debug.h"
#include "util/symbol.h"
#include "shell/dir.h"
#include "shell/shell.h"
#include "shell/signals.h"
#include "shell/redir.h"

extern char **environ;

int Exit(LinkedList *list)
{
  doutput("Exiting shell.\n");
  deInitShell();
  if(list) exit(atoi(list->s));
  else exit(RET_SUCCESS);
}

char *getPrompt(void)
{
  char *prompt;
  LinkedList *pwd;
  char *tmp;

  pwd = getpwd();
  tmp = VarValue("PWD");
  if(tmp == NULL) tmp = dir_ll_to_str(pwd);
  prompt = emalloc(strlen(tmp) + 3);
  *prompt = 0;
  sprintf(prompt, "%s> ", tmp);
  free(tmp);
  return prompt;
}

void initShell(void)
{
  char **env;
  char *var,*val,*p;
  char *dir,*tmp,*rcfile;
  FILE *fp;
  LinkedList *_tmp_ll;

  env = environ;

  while(*env != NULL) {
    var = mystrdup(*env);
    p = strchr(var, '=');
    val = mystrdup(p+1);
    *p = 0;
    AssignVar(var,val);
    free(var);
    free(val);
    env++;
  }

  if(!VarExists("__DEBUG")) {
    AssignVar("__DEBUG","0");
  }
  if(!VarExists("YYDEBUG")) {
    AssignVar("YYDEBUG","0");
  }

  dir = VarValue("PWD");
  if(dir == NULL) {
    dir = emalloc(5000);
    getcwd(dir,5000);  
  }
  setpwd(NULL);  // FIXME: Should use cwd to do this???
  chdir("/");
  _tmp_ll = createll(dir);
  _builtin_cwd(_tmp_ll);
  free(_tmp_ll);
  free(dir);
 
  initSignalHandling(); 

  tmp = VarValue("HOME");
  AssignVar("PAD", tmp);
  rcfile = emalloc(strlen(tmp) + 12);
  sprintf(rcfile, "%s/.pr0nshrc", tmp);
  free(tmp);
  if((fp = fopen(rcfile,"rt")) != NULL) execFromFP(fp);
  free(rcfile);
}

// FIXME: Should have a better way of freeing memory from _pwd
void deInitShell(void)
{
//  LinkedList *_tmp_ll;

//  RemoveAllVars();
//  _tmp_ll = createll("/");
//  cwd(_tmp_ll);
//  freell(_tmp_ll);
}
