/* signals.c -- Signal handling functions. */

/* Copyright (C) 2003 Mike Thielman 
 *
 * This file is part of pr0nsh, the Porn Shell.
 *
 * Pr0nsh is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2, or (at your option) any later
 * version.
 *
 * Pr0nsh is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Pr0nsh; see the file COPYING.  If not, write to the Free Software
 * Foundation, 59 Temple Place, Suite 330, Boston, MA 02111 USA. */

#include <signal.h>
#include "shell/signals.h"
#include "util/debug.h"

void _SIGINT_catch(int n);

char *getSigName(int signal)
{
  int i = 0;
  for(; signals[i].signal != 0; i++)
  {
    if(signals[i].signal == signal) return signals[i].name;
  }
}

char *getSigDesc(int signal)
{
  int i = 0;
  for(; signals[i].signal != 0; i++)
  {
    if(signals[i].signal == signal) return signals[i].desc;
  }
}

void _SIGPIPE_catch(int n)
{
  doutput("Caught signal SIGPIPE\n");
  if (signal(SIGPIPE, _SIGPIPE_catch) == SIG_ERR)
    errormsg("error catching SIGPIPE", NULL);
}

void _SIGTTIN_catch(int n)
{
  doutput("Caught signal SIGTTIN\n");
  if (signal(SIGTTIN, _SIGTTIN_catch) == SIG_ERR)
    errormsg("error catching SIGTTIN", NULL);
}

void _SIGTTOU_catch(int n)
{
  doutput("Caught signal SIGTTOU\n");
  if (signal(SIGTTOU, _SIGTTOU_catch) == SIG_ERR)
    errormsg("error catching SIGTTOU", NULL);
}

void _SIGINT_catch(int n)
{
  if (signal(SIGINT, _SIGINT_catch) == SIG_ERR)
    errormsg("error catching SIGINT", NULL);
}

void _SIGCHLD_catch(int n)
{
  if (signal(SIGCHLD, _SIGCHLD_catch) == SIG_ERR)
    errormsg("error catching SIGCHLD", NULL);
  SIGCHLDHandler();
}

void _SIGTSTP_catch(int n)
{
  doutput("Received SIGTSTP\n");
  if (signal(SIGTSTP, _SIGTSTP_catch) == SIG_ERR)
    errormsg("error catching SIGTSTP", NULL);
  SIGTSTPHandler();
}

void initSignalHandling(void)
{
  if (signal(SIGCHLD, _SIGCHLD_catch) == SIG_ERR)
    errormsg("error catching SIGCHLD", NULL);
  if (signal(SIGINT, _SIGINT_catch) == SIG_ERR)
    errormsg("error catching SIGINT", NULL);
  if (signal(SIGTSTP, _SIGTSTP_catch) == SIG_ERR)
    errormsg("error catching SIGTSTP", NULL);
  if (signal(SIGPIPE, _SIGPIPE_catch) == SIG_ERR)
    errormsg("error catching SIGPIPE", NULL);
  if (signal(SIGTTIN, _SIGTTIN_catch) == SIG_ERR)
    errormsg("error catching SIGTTIN", NULL);
  if (signal(SIGTTOU, _SIGTTOU_catch) == SIG_ERR)
    errormsg("error catching SIGTTOU", NULL);
}
