/* llist.c -- Functions for dealing with linked lists of strings */

/* Copyright (C) 2003 Mike Thielman 
 *
 * This file is part of pr0nsh, the Porn Shell.
 *
 * Pr0nsh is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2, or (at your option) any later
 * version.
 *
 * Pr0nsh is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Pr0nsh; see the file COPYING.  If not, write to the Free Software
 * Foundation, 59 Temple Place, Suite 330, Boston, MA 02111 USA. */

#include <string.h>
#include <stdlib.h>
#include "util/debug.h"
#include "util/llist.h"

void freell(LinkedList *list)
{
  LinkedList *cur = list;

  if(list == NULL) return; 
  freell(list->next);
  free(list->s);
  free(list);
}

LinkedList *createll(char *s)
{
  LinkedList *lp;

  lp = newll();
  lp->s = mystrdup(s);
  lp->next = (LinkedList *) 0;
  return lp;
}

LinkedList *copyLinkedList(LinkedList *src)
{
  LinkedList *ptr;
  LinkedList *dest = NULL;

  ptr = src;
  if(src == NULL) { return NULL; }
  else {
    do {
      dest = pushll(dest,ptr->s);
      ptr = ptr->next;
    } while(ptr != NULL);
  }
  return dest;
}

int searchll(LinkedList *list, char *s)
{
  LinkedList *cur = list;

  while(cur != NULL) {
    if(!strcmp(cur->s, s)) return 1;
    cur = cur->next;
  }
  return 0;
}

LinkedList *pushll(LinkedList *list,char *s)
{
  char *str;
  LinkedList *cur;

  if(list == NULL) { list = createll(s); return list; }
  else {
    cur = list;
    while(cur->next != NULL) { cur = cur->next; }
    cur->next = createll(s);
    return list;
  }
}

LinkedList *popll(LinkedList *list)
{
  char *str;
  LinkedList *prev,*cur;

  if(list == NULL) return NULL;
  else if(list->next == NULL) {
    free(list->s);
    free(list);
    return NULL;
  }
  else {
    cur = list;
    while(cur->next != NULL) { prev = cur;
                               cur = cur->next; }

    free(prev->next->s);
    free(prev->next);
    prev->next = NULL;
    return list;    
  }
}

char *ll_to_charp(LinkedList *list)
{
  LinkedList *arg = list;
  char *str;
  int len = 0;
  
  if(list == NULL) return NULL;

  while(arg != NULL) {
    len += (strlen(arg->s) + 1);
    arg = arg->next;
  }
  str = emalloc(sizeof(char) * (len-1));
 
  *str = 0; 
  arg = list;
  while(arg != NULL) {
    strcat(str, arg->s);
    if(arg->next != NULL) strcat(str, " ");
    arg = arg->next;
  }
  return str;
}

void printll(LinkedList *list)
{
  LinkedList *arg = list;

  while(arg != NULL) {
    doutput("%s\n", arg->s);
    arg = arg->next;
  }
}

char **ll_to_charpp(LinkedList *list)
{
  char **array;
  int i = 0;
  LinkedList *cur = list;

  array = emalloc(sizeof(char *) * numEntriesll(list));
  while(cur != NULL) {
    array[i] = mystrdup(cur->s);
    i++;
    cur = cur->next;
  }
  array[i] = NULL;
  return array;
}

int numEntriesll(LinkedList *list)
{
  int n = 1;
  LinkedList *arg = list;
  if(arg == NULL) return 0;
  for(; arg->next != NULL; arg=arg->next,n++);
  return n;
}

LinkedList *getNextll(LinkedList *list)
{
  if(list) return list->next;
  return NULL;
}

char *getValuell(LinkedList *list)
{
  if(list) return list->s;
  return NULL;
}

LinkedList *newll(void)
{
  LinkedList *ll;

  ll = (LinkedList *) emalloc(sizeof(LinkedList));
  ll->s = NULL;
  ll->next = NULL;

  return ll;
}
